# This file is loaded when test execution from TestWise
require 'rspec/core/formatters/base_formatter'
require 'rspec/core/formatters/console_codes'
require 'rspec/core/formatters/base_text_formatter'
require 'rspec/core/formatters/progress_formatter'
require 'logger'
require 'json'

require "#{File.dirname(__FILE__)}/testwise_listener.rb"
require "#{File.dirname(__FILE__)}/testwise_runtime_support.rb"

# class TestWiseFormatter < RSpec::Core::Formatters::BaseFormatter
class TestWiseRSpecFormatter < RSpec::Core::Formatters::BaseTextFormatter

  include TestWiseRuntimeSupport
  
  # This registers the notifications this formatter supports, and tells
  # us that this was written against the RSpec 3.x formatter API.
  RSpec::Core::Formatters.register self, :example_started, :example_failed, :example_passed

  def initialize(output)
    super(output)
    # @logger = ::Logger.new('/Users/zhimin/tmp/testwise_rspec_formamtter.log')
    
    ::TestWise::Runtime.load_webdriver_support 
    ::TestWise::Runtime.load_watir_support 
       
	end

	# This method is invoked before any examples are run, right after
  def start(notification)
    super
    # @logger.info "[TestWiseRSpecFormatter] Total test cases: #{@example_count}"
    hash = {}
    hash["test_case_count"] = @example_count
    json_data = hash.to_json
    connect_to_testwise("START",  json_data + "\r\n")
  end

	# called at very end
  def close(notification)
    super
    # @logger.info "[TestWiseRSpecFormatter] Close"
    json_data = {}.to_json
    connect_to_testwise("CLOSE",  json_data + "\r\n")
  end

  def example_group_started(notification)
    super
    # @logger.info "[TestWiseRSpecFormatter] Running example group: #{notification.group}"
  end

  def example_started(notification)
    test_case_file_path =  notification.example.metadata[:file_path]
    test_case_name = notification.example.description.to_s
    # @logger.info "[TestWiseRSpecFormatter] about to run #{test_case_file_path}"
    hash = {}
    hash[:file_path] = test_case_file_path
    hash[:test_case] = test_case_name
    json_data = hash.to_json
    connect_to_testwise("TESTSTART",  json_data + "\r\n")
	end

	def example_passed(notification)
    output.print RSpec::Core::Formatters::ConsoleCodes.wrap('.', :success)
    test_case_file_path =  notification.example.metadata[:file_path]
    test_case_name = notification.example.description.to_s
    hash = {}
    hash[:file_path] = test_case_file_path
    hash[:test_case] = test_case_name
    json_data = hash.to_json
    # @logger.info "[TestWiseRSpecFormatter]  #{json_data.inspect} passed"
    connect_to_testwise("TESTPASS",  json_data + "\r\n")
  end

  def example_pending(notification)
    # @logger.info "[TestWiseRSpecFormatter]  Pending: #{example}"
    output.print RSpec::Core::Formatters::ConsoleCodes.wrap('*', :pending)
  end

  def example_failed(failure)
    # FailedExampleNotification
    output.print RSpec::Core::Formatters::ConsoleCodes.wrap('F', :failure)
    # puts "[TestWiseFormatter] Example failed: " + notification.inspect
    # exception = notification.metadata[:execution_result][:exception]

    error_test_case_name = failure.example.description
    error_message = failure.exception.message
    hash = {}
    hash[:file_path] = failure.example.metadata[:file_path]
    hash[:test_case] = error_test_case_name.to_s

    # NOTE: just send mimimal info to TestWise, the output analyse will be done after execution complete 
    
=begin
    # prevent too big
    if error_message.size > 2000
      error_message = error_message[0..1744] + "..." + error_message[-256..-1]
    end
    hash[:error_message] = error_message

     error_backtrace_array = failure.exception.backtrace
     error_backtrace = error_backtrace_array.select{|x| !x.include?("/gems/rspec")} # exclude rpsec back trace
     error_backtrace = error_backtrace.join("\r\n")
 
    if error_backtrace.size > 2000
      error_backtrace = error_backtrace[0..1000] + "..." + error_backtrace[-1000..-1]
    end
    hash[:error_backtrace] = error_backtrace.to_json
=end

    json_data = hash.to_json
    #@logger.info "[TestWiseRSpecFormatter]  failed."
    #@logger.info "[TestWiseRSpecFormatter]  #{json_data.inspect}"
    connect_to_testwise("TESTFAIL",  json_data + "\r\n")
  end

=begin
  def connect_to_testwise (message_type, body)
      # TODO, wait until getting socket working
     the_message = message_type + "|" + body
     @logger.info("[MESSAGE]" + the_message);
     Thread.pass
     begin
       if @last_message == the_message then # ignore the message same as preivous one
         return
       end
       testwise_port = ENV["TESTWISE_TRACE_PORT"].to_i || $TESTWISE_TRACE_PORT
       if testwise_port < 1024
         testwise_port = 7535 # default
       end
       @logger.info("Try to connect to port: #{testwise_port}")
       testwise_socket = Socket.new(Socket::AF_INET, Socket::SOCK_STREAM, 0)
       testwise_socket.connect(Socket.pack_sockaddr_in(testwise_port, '127.0.0.1'))
       testwise_socket.puts(the_message)
       @last_message = the_message
       testwise_socket.close
     rescue => e
       @logger.warn("Failed connect to testwise: ")
       @logger.warn(e)
     end
     
   end
=end

end
