import unittest
import xmlrunner
import time
import datetime
import sys
import os
from selenium import webdriver
from selenium.webdriver.support.ui import Select
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC

sys.path.insert(0, os.path.dirname(os.path.realpath(__file__)) + "/../")
from test_helper import TestHelper
from pages.login_page import LoginPage
from pages.flight_page import FlightPage

class FlightTestCase(unittest.TestCase, TestHelper):

  @classmethod
  def setUpClass(cls):
    if os.environ.get('BROWSER') == "firefox":
      cls.driver = webdriver.Firefox()
    else:
      cls.driver = webdriver.Chrome()
    cls.save_driver_session()
    cls.driver.set_window_size(1028, 720)
    cls.driver.set_window_position(30, 78)

    cls.driver.get(cls.site_url())
    login_page = LoginPage(cls.driver)
    login_page.enter_username("agileway")
    login_page.enter_password("testwise")
    login_page.click_sign_in()

  @classmethod
  def tearDownClass(cls):
    time.sleep(1)
    if not cls.is_debugging():
      cls.driver.quit()

  def setUp(self):
    self.driver.get(self.site_url())

  def test_select_return_flight(self):
    flight_page = FlightPage(self.driver)
    flight_page.select_trip_type("return")
    flight_page.select_depart_from("Sydney")
    flight_page.select_arrive_at("New York")

    flight_page.select_depart_day("02")
    flight_page.select_depart_month("May 2023")
    flight_page.select_return_day("04")
    flight_page.select_return_month("June 2023")
    flight_page.click_continue()
    self.assertIn("2023-05-02 Sydney to New York", self.driver.find_element(By.TAG_NAME, "body").text)
    self.assertIn("2023-06-04 New York to Sydney", self.driver.find_element(By.TAG_NAME, "body").text)

  def test_select_oneway_flight(self):
    flight_page = FlightPage(self.driver)
    flight_page.select_trip_type("oneway")
    flight_page.select_depart_from("Sydney")
    flight_page.select_arrive_at("New York")
    flight_page.select_depart_day("02")
    flight_page.select_depart_month("November 2023")
    flight_page.click_continue()
    self.assertIn("2023-11-02 Sydney to New York", self.driver.find_element(By.TAG_NAME, "body").text)

# if __name__ == '__main__':
#     unittest.main(
#         testRunner=xmlrunner.XMLTestRunner(output='reports'),
#         failfast=False, buffer=False, catchbreak=False)
